import chalk from "chalk";
import os from "os";
import path from "path";
import { diffTreesToConfig, FileTree } from "./tree/functions.js";
const dataFromStdIn = () => new Promise((resolve) => process.stdin.once("data", resolve));
export const doWith = async (params, config) => {
    process.stdout.write(`${chalk.yellow("Reading file system, please wait...")}\r`);
    const preTree = new FileTree(process.cwd(), path.join(process.env.C7_ENV === "development" ? process.cwd() : os.tmpdir(), `.c7.pre`));
    await preTree.parseFiles();
    console.log(`${chalk.green(`Starting to record...${" ".repeat(process.stdout.columns - 21)}\nMake changes to your files, then ${chalk.bold("press Enter")} to stop recording.`)}`);
    await dataFromStdIn();
    console.log(`${chalk.green("Stopped recording!")}`);
    const postTree = new FileTree(process.cwd(), path.join(process.env.C7_ENV === "development" ? process.cwd() : os.tmpdir(), `.c7.post`));
    await postTree.parseFiles();
    console.log(`${chalk.yellow("Calculating diffs...")}`);
    await diffTreesToConfig(preTree, postTree, params, config);
    process.stdin.pause();
};
