import { ArgsError } from "./errors.js";
import { isValidAction, ValidActions } from "./types.js";
const checkDoubleDash = (arg) => {
    if (arg.slice(0, 2) !== "--") {
        throw new ArgsError(`unexpected argument ${arg}`);
    }
};
const ArgsReducerBuilder = (options) => {
    const AllowVars = options?.AllowVars || false;
    return (args) => {
        const optionValueMap = new Map();
        return args.reduce((map, curr) => {
            checkDoubleDash(curr);
            const mappedArg = curr.slice(2).split("=");
            optionValueMap.set(mappedArg[0], mappedArg[1]);
            if (AllowVars) {
                let value = mappedArg[1];
                let openBracket = value.indexOf("[");
                let closedBracket = value.indexOf("]", openBracket);
                while (closedBracket > openBracket && openBracket != -1) {
                    const capturedGroup = value.substring(openBracket + 1, closedBracket);
                    if (optionValueMap.has(capturedGroup)) {
                        value =
                            value.slice(0, openBracket) +
                                optionValueMap.get(capturedGroup) +
                                value.slice(closedBracket + 1);
                    }
                    openBracket = value.indexOf("[", closedBracket);
                    closedBracket = value.indexOf("]", openBracket);
                }
                mappedArg[1] = value;
            }
            map.push(mappedArg);
            return map;
        }, []);
    };
};
export const parseArgs = (config, argv) => {
    const args = argv.slice(2);
    if (args.length === 0) {
        throw new ArgsError(`not enough parameters; wanted >0, got 0`);
    }
    const action = args.shift();
    if (!action || !isValidAction(action)) {
        throw new ArgsError(`invalid action; wanted one of [${ValidActions.join(", ")}], got "${action}"`);
    }
    const params = {
        action,
        id: "default",
        optionValues: [],
    };
    if (args[0].indexOf("--") !== 0) {
        params.id = args[0];
        args.shift();
    }
    checkDoubleDash(args[0]);
    const argsReducer = ArgsReducerBuilder(config?.options);
    params.optionValues = argsReducer(args);
    return params;
};
